@@ -239,7 +239,7 @@ class ReadingTest(BaseTest):
             i, get_cover=True, cover_as_data=True) for i in range(1, 4)}
         cache = None
         for mi2, mi1 in zip(list(new_metadata.values()), list(old_metadata.values())):
-            self.compare_metadata(mi1, mi2)
+            self.compare_metadata(mi1, mi2, exclude=('pages',))
     # }}}
 
     def test_serialize_metadata(self):  # {{{
@@ -253,9 +253,9 @@ class ReadingTest(BaseTest):
         for i in range(1, 4):
             mi = cache.get_metadata(i, get_cover=True, cover_as_data=True)
             rmi = msgpack_loads(msgpack_dumps(mi))
-            self.compare_metadata(mi, rmi, exclude='format_metadata has_cover formats id'.split())
+            self.compare_metadata(mi, rmi, exclude='format_metadata has_cover formats id pages'.split())
             rmi = json_loads(json_dumps(mi))
-            self.compare_metadata(mi, rmi, exclude='format_metadata has_cover formats id'.split())
+            self.compare_metadata(mi, rmi, exclude='format_metadata has_cover formats id pages'.split())
     # }}}
 
     def test_get_cover(self):  # {{{
@@ -465,6 +465,20 @@ class ReadingTest(BaseTest):
 
     # }}}
 
+    def test_by_year(self):
+        ' Test grouping of books by date fields '
+        cache = self.init_cache()
+        self.assertEqual(cache.books_by_year(), {2011: {1, 2, 3}})
+        self.assertEqual(cache.books_by_year(restrict_to_books={1,2}), {2011: {1, 2}})
+        self.assertEqual(cache.books_by_month(), {(2011, 8): {1}, (2011, 9): {2, 3}})
+        self.assertEqual(cache.books_by_month(restrict_to_books={1,2}), {(2011, 8): {1}, (2011, 9): {2}})
+        self.assertEqual(cache.books_by_year(restrict_to_books=range(cache.backend.max_number_of_variables+200)), {2011: {1, 2, 3}})
+        cache.set_field('#date', {3:datetime.datetime(2001, 11, 10).astimezone(utc_tz)})
+        self.assertEqual(cache.books_by_year('#date'), {2011: {1, 2}, 2001:{3}})
+        self.assertEqual(cache.books_by_year('#date', restrict_to_books={1}), {2011: {1}})
+        self.assertEqual(cache.books_by_month('#date'), {(2011, 9): {1, 2}, (2001, 11): {3}})
+        self.assertEqual(cache.books_by_month('#date', restrict_to_books={1,3}), {(2011, 9): {1}, (2001, 11): {3}})
+
     def test_author_sort_for_authors(self):  # {{{
         'Test getting the author sort for authors from the db'
         cache = self.init_cache()
@@ -630,7 +644,7 @@ class ReadingTest(BaseTest):
                     except Exception:
                         pass
                     return x
-                if field not in {'#series_index'}:
+                if field != '#series_index':
                     v = pmi.get_standard_metadata(field)
                     self.assertTrue(v is None or isinstance(v, dict))
                     self.assertEqual(f(mi.get_standard_metadata(field, False)), f(v),
@@ -820,15 +834,36 @@ class ReadingTest(BaseTest):
 
         db = self.init_cache(self.library_path)
         db.create_custom_column('mult', 'CC1', 'composite', True, display={'composite_template': 'b,a,c'})
+        db.create_custom_column('pages', 'Pages', 'int', False)
 
         # need an empty metadata object to pass to the formatter
         db = self.init_legacy(self.library_path)
+        db.new_api.set_field('#pages', {1: '11', 2: '22', 3: '33'})
         mi = db.get_metadata(1)
 
+        # test width_from_pages
+        v = formatter.safe_format('{#pages:width_from_pages(2,2,0.5)}', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, '1.0')
+        v = formatter.safe_format('{#pages:width_from_pages()}', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(int(float(v) * 1000), 26)
+        v = formatter.safe_format('{#pages:width_from_pages(1500)}', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(int(float(v) * 1000), 26)
+        v = formatter.safe_format('{#pages:width_from_pages(1500,2)}', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(int(float(v) * 1000), 26)
+        v = formatter.safe_format('{#pages:width_from_pages(1500,2,11)}', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(int(float(v) * 1000), 26)
+
         # test counting books matching the search
         v = formatter.safe_format('program: book_count("series:true", 0)', {}, 'TEMPLATE ERROR', mi)
         self.assertEqual(v, '2')
 
+        # test python templates
+        v = formatter.safe_format('python:\ndef evaluate(book, ctx): return "x"', {}, 'TEMPLATE ERROR', mi)
+        if os.environ.get('CALIBRE_ALLOW_PYTHON_TEMPLATES', '1') == '1':
+            self.assertEqual(v, 'x')
+        else:
+            self.assertTrue(v.startswith('TEMPLATE ERROR'))
+
         # test counting books when none match the search
         v = formatter.safe_format('program: book_count("series:afafaf", 0)', {}, 'TEMPLATE ERROR', mi)
         self.assertEqual(v, '0')
@@ -954,3 +989,7 @@ def evaluate(book, ctx):
         unload_user_template_functions('aaaaa')
         self.assertEqual(set(v.split(',')), {'Tag One', 'News', 'Tag Two', 'one argument'})
     # }}}
+
+    def test_cover_cache(self):
+        from calibre.gui2.library.caches import test_cover_cache
+        test_cover_cache(self)
